window.RPLP = window.RPLP || {};

(function() {
  /**
   * YUI Library aliases
   */
  var Dom = YAHOO.util.Dom, Event = YAHOO.util.Event, Selector = YAHOO.util.Selector, Bubbling = YAHOO.Bubbling;

  /**
   * Alfresco Slingshot aliases
   */
  var $html = Alfresco.util.encodeHTML, $links = Alfresco.util.activateLinks, $combine = Alfresco.util.combinePaths, $userProfile = Alfresco.util.userProfileLink;

  Alfresco.component.ExtDataListToolbar = function(htmlId) {
    return Alfresco.component.ExtDataListToolbar.superclass.constructor.call(this, htmlId);
  };

  /**
   * Extend Alfresco.component.DataGrid
   */
  YAHOO.extend(Alfresco.component.ExtDataListToolbar, Alfresco.component.DataListToolbar);

  /**
   * Augment prototype with Common Actions module
   */
  YAHOO.lang.augmentProto(Alfresco.component.ExtDataListToolbar, Alfresco.service.DataListActions);

  /**
   * Augment prototype with main class implementation, ensuring overwrite is
   * enabled
   */
  YAHOO.lang.augmentObject(Alfresco.component.ExtDataListToolbar.prototype, {
    /**
     * Fired by YUI when parent element is available for scripting.
     * 
     * @method onReady
     */
    onReady : function DataListToolbar_onReady() {
      this.widgets.exportCsvButton = Alfresco.util.createYUIButton(this, "exportCsvButton", this.onExportCsv, {
        disabled : true
      });
      this.widgets.newVoButton = Alfresco.util.createYUIButton(this, "newVoButton", this.onNewVO, {
        disabled: true,
        value : "createVO"
      });
      this.widgets.newVorButton = Alfresco.util.createYUIButton(this, "newVorButton", this.onNewVOR, {
        disabled: true,
        value : "createVOR"
      });

      if (Alfresco.util.getQueryStringParameter("list", document.location.href) == null) {
        // If no list is selected hide these two buttons, so it only displays the disabled "New Item" button
        this.widgets.newVoButton.destroy();
        this.widgets.newVorButton.destroy();
      }

      Bubbling.on("activeDataListChanged", this.onActiveDataListChanged, this);
      Alfresco.component.ExtDataListToolbar.superclass.onReady.call(this);
    },

    onActiveDataListChanged : function ExtDatalistToolbar_onActiveDataListChanged(layer, args) {
      var obj = args[1];
      if ((obj !== null) && (obj.dataList !== null)) {
        var dlType = obj.dataList.itemType;
        if ("lysedl:variationOrderRequestItem" == obj.dataList.itemType) {
          this.widgets.newRowButton.destroy();
        } else {
          this.widgets.newVoButton.destroy();
          this.widgets.newVorButton.destroy();
        }
      }
    },

    /**
     * Export CSV button click handler
     * 
     * @method onExportCsv
     * @param e
     *            {object} DomEvent
     * @param p_obj
     *            {object} Object passed back from addListener method
     */
    onExportCsv : function ExtDataListToolbar_onExportCsv(e, p_obj) {
      var nodeRef = Alfresco.util.NodeRef(this.modules.dataGrid.datalistMeta.nodeRef)
      window.location.href = Alfresco.constants.PROXY_URI + "/fme/slingshot/datalists/list/node/" + nodeRef.uri + "?format=xlsx";

    },

    displayCreateForm : function(e, p_obj, title, templateUrl) {
      var datalistMeta = this.modules.dataGrid.datalistMeta, destination = datalistMeta.nodeRef, itemType = datalistMeta.itemType;

      // Intercept before dialog show
      var doBeforeDialogShow = function DataListToolbar_onNewVO_doBeforeDialogShow(p_form, p_dialog) {
        Alfresco.util.populateHTML([ p_dialog.id + "-dialogTitle", this.msg(title) ]);
      };

      // Using Forms Service, so always create new instance
      var createRow = new Alfresco.module.SimpleDialog(this.id + "-createRow");

      createRow.setOptions({
        width : this.options.newItemDialogWidth,
        templateUrl : templateUrl,
        actionUrl : null,
        destroyOnHide : true,
        doBeforeDialogShow : {
          fn : doBeforeDialogShow,
          scope : this
        },
        onSuccess : {
          fn : function DataListToolbar_onNewVO_success(response) {
            YAHOO.Bubbling.fire("dataItemCreated", {
              nodeRef : response.json.persistedObject
            });

            Alfresco.util.PopupManager.displayMessage({
              text : this.msg("message.new-row.success")
            });
          },
          scope : this
        },
        onFailure : {
          fn : function DataListToolbar_onNewVO_failure(response) {
            Alfresco.util.PopupManager.displayMessage({
              text : this.msg("message.new-row.failure")
            });
          },
          scope : this
        }
      }).show();

    },

    /**
     * New VO button click handler
     * 
     * @method onNewVO
     * @param e
     *            {object} DomEvent
     * @param p_obj
     *            {object} Object passed back from addListener method
     */
    onNewVO : function DataListToolbar_onNewVO(e, p_obj) {
      var datalistMeta = this.modules.dataGrid.datalistMeta, destination = datalistMeta.nodeRef, itemType = datalistMeta.itemType;

      this.displayCreateForm(e, p_obj, "label.new-vo.title", YAHOO.lang.substitute(Alfresco.constants.URL_SERVICECONTEXT + "components/form?vorType=vo&itemKind={itemKind}&itemId={itemId}&formId={formId}&destination={destination}&mode={mode}&submitType={submitType}&showCancelButton=true", {
        itemKind : "type",
        itemId : itemType,
        formId : "variationOrder",
        destination : destination,
        mode : "create",
        submitType : "json"
      }));
    },

    /**
     * New VOR button click handler (basically same as onNewRow)
     * 
     * @method onNewVOR
     * @param e
     *            {object} DomEvent
     * @param p_obj
     *            {object} Object passed back from addListener method
     */
    onNewVOR : function DataListToolbar_onNewVOR(e, p_obj) {
      var datalistMeta = this.modules.dataGrid.datalistMeta, destination = datalistMeta.nodeRef, itemType = datalistMeta.itemType;

      this.displayCreateForm(e, p_obj, "label.new-vor.title", YAHOO.lang.substitute(Alfresco.constants.URL_SERVICECONTEXT + "components/form?itemKind={itemKind}&itemId={itemId}&destination={destination}&mode={mode}&submitType={submitType}&showCancelButton=true", {
        itemKind : "type",
        itemId : itemType,
        destination : destination,
        mode : "create",
        submitType : "json"
      }));
    },

    /**
     * Create meeting minutes click handler
     * 
     * @method onActionCreateMeetingMinutes
     * @param e
     *            {object} DomEvent
     * @param p_obj
     *            {object} Object passed back from addListener method
     */
    onActionCreateMeetingMinutes : function ExtDataListToolbar_onActionCreateMeetingMinutes(e, p_obj) {
      // Collect nodes to operate on.
      var actionListRefs = [];
      for (var i = 0, j = e.length; i < j; i++) {
        var row = e[i];
        actionListRefs.push(row.nodeRef);
       }
      window.RPLP.globalActionListRefs = actionListRefs;

      var datalistMeta = this.modules.dataGrid.datalistMeta, destination = datalistMeta.nodeRef;

      // Intercept before dialog show
      var doBeforeDialogShow = function dlA_onActionDetails_doBeforeDialogShow(p_form, p_dialog) {
        Alfresco.util.populateHTML([ p_dialog.id + "-dialogTitle", this.msg("form.createMeetingMinutes.title") ]);
      };

      var templateUrl = YAHOO.lang.substitute(Alfresco.constants.URL_SERVICECONTEXT + "components/form?itemKind={itemKind}&itemId={itemId}&destination={destination}&mode={mode}&submitType={submitType}&formId={formId}&showCancelButton=true", {
        itemKind : "type",
        itemId : "lyse:meetingMinutes",
        mode : "create",
        destination : destination,
        submitType : "json",
        formId : "model-type"
      });

      // Using Forms Service, so always create new instance
      var createMeetingMinutes = new Alfresco.module.SimpleDialog(this.id + "-createMeetingMinutes");

      createMeetingMinutes.setOptions({
        width : this.options.newItemDialogWidth,
        templateUrl : templateUrl,
        actionUrl : null,
        destroyOnHide : true,
        doBeforeDialogShow : {
          fn : doBeforeDialogShow,
          scope : this
        },
        onSuccess : {
          fn : function ExtDataListToolbar_onActionCreateMeetingMinutes_success(response) {
            window.location.href = Alfresco.constants.URL_CONTEXT + "page/site/" + encodeURIComponent(Alfresco.constants.SITE) + '/document-details?nodeRef=' + encodeURIComponent(response.json.persistedObject)
          },
          scope : this
        },
        onFailure : {
          fn : function ExtDataListToolbar_onActionCreateMeetingMinutes_failure(response) {
            Alfresco.util.PopupManager.displayMessage({
              text : this.msg("message.createMeetingMinutes.failure" + response.serverResponse.responseText)
            });
          },
          scope : this
        }
      }).show();

    },
    /**
     * Submit for review
     * 
     * @method onActionSubmitForReview
     * @param e
     *            {object} DomEvent
     * @param p_obj
     *            {object} Object passed back from addListener method
     */
    onActionSubmitForReview : function ExtDataListToolbar_onActionSubmitForReview(e, p_obj) {
      // Collect nodes to operate on.
      var nodes = [];
      for (var i = 0, j = e.length; i < j; i++) {
        var row = e[i];
        var nodeRef = {
          nodeRef : row.nodeRef
        };
        nodes.push(nodeRef);
      }

      Alfresco.util.Ajax.jsonPost({
        url : Alfresco.constants.PROXY_URI + "/lyse/api/start-workflow",
        responseContentType : "application/json",
        dataObj : nodes,
        successCallback : {
          fn : function ExtDataListToolbar_onActionSubmitForReview_success(response) {

            Alfresco.util.PopupManager.displayMessage({
              text : this.msg("actions.submit-for-review-success")
            });

          },
          scope : this
        },
        failureCallback : {
          fn : function ExtDataListToolbar_onActionSubmitForReview_failure(response) {
            Alfresco.util.PopupManager.displayMessage({
              text : response.json.message
            });
          },
          scope : this
        }
      });

    },

    /**
     * New Row button click handler
     * 
     * @method onNewRow
     * @param e
     *            {object} DomEvent
     * @param p_obj
     *            {object} Object passed back from addListener method
     */
    onNewRow : function ExtDataListToolbar_onNewRow(e, p_obj) {
      var datalistMeta = this.modules.dataGrid.datalistMeta, destination = datalistMeta.nodeRef, itemType = datalistMeta.itemType;

      // Intercept before dialog show
      var doBeforeDialogShow = function DataListToolbar_onNewRow_doBeforeDialogShow(p_form, p_dialog) {
        Alfresco.util.populateHTML([ p_dialog.id + "-dialogTitle", this.msg("label.new-row.title") ], [ p_dialog.id + "-dialogHeader", this.msg("label.new-row.header") ]);
      };

      var templateUrl = YAHOO.lang.substitute(Alfresco.constants.URL_SERVICECONTEXT + "components/form?itemKind={itemKind}&itemId={itemId}&destination={destination}&mode={mode}&submitType={submitType}&showCancelButton=true", {
        itemKind : "type",
        itemId : itemType,
        destination : destination,
        mode : "create",
        submitType : "json"
      });

      // Using Forms Service, so always create new instance
      var createRow = new Alfresco.module.SimpleDialog(this.id + "-createRow");

      createRow.setOptions({
        width : "800px",
        templateUrl : templateUrl,
        actionUrl : null,
        destroyOnHide : true,
        doBeforeDialogShow : {
          fn : doBeforeDialogShow,
          scope : this
        },
        onSuccess : {
          fn : function DataListToolbar_onNewRow_success(response) {
            YAHOO.Bubbling.fire("dataItemCreated", {
              nodeRef : response.json.persistedObject
            });

            Alfresco.util.PopupManager.displayMessage({
              text : this.msg("message.new-row.success")
            });
          },
          scope : this
        },
        onFailure : {
          fn : function DataListToolbar_onNewRow_failure(response) {
            Alfresco.util.PopupManager.displayMessage({
              text : this.msg("message.new-row.failure")
            });
          },
          scope : this
        }
      }).show();
    }

  }, true);
})();
