/**
 * @module lyseproject/ProjectService
 * @extends module:alfresco/core/Core
 * @mixes module:alfresco/core/CoreXhr
 * @author Rasmus Melgaard
 * 
 */
// Define module requirements and callback...
define(["dojo/_base/declare",
        "alfresco/core/Core",
        "alfresco/core/CoreXhr",
        "alfresco/core/NotificationUtils",
        "dojo/request/xhr",
        "dojo/json",
        "dojo/_base/lang",
        "dojo/dom-construct",
        "alfresco/buttons/AlfButton"],
        function(declare, AlfCore, AlfXhr, NotificationUtils, xhr, JSON, lang, domConstruct, AlfButton) {
   
   return declare([AlfCore, AlfXhr, NotificationUtils], {
 
      constructor: function(args) {
        this.alfSubscribe("ALF_CREATE_PROJECT", lang.hitch(this, "createProject"));
        this.alfSubscribe("LYSE_EDIT_PROJECT", lang.hitch(this, "editProject"));

        this.alfSubscribe("ALF_CREATE_CONTRACTOR", lang.hitch(this, "createContractor"));
        this.alfSubscribe("LYSE_EDIT_CONTRACTOR", lang.hitch(this, "editContractor"));
        /*
        //Ensure the create-project.js javascript is included (Based on create-site.js)
        var _this = this;
         require([Alfresco.constants.URL_RESCONTEXT + "modules/lyse-create-project.js"], function() {
            _this.alfLog("log", "Create Project JavaScript resource loaded");
         });
         require([Alfresco.constants.URL_RESCONTEXT + "components/form/date.js"], function() {
            _this.alfLog("log", "Date resource loaded");
         });
         require([Alfresco.constants.URL_RESCONTEXT + "components/form/date-picker.js"], function() {
            _this.alfLog("log", "Date picker resource loaded");
         }); 
         require([Alfresco.constants.URL_RESCONTEXT + "components/object-finder/object-finder.js"], function() {
            _this.alfLog("log", "Object finder resource loaded");
         });  
                       
        */
      },

      /**
       * This method delegates site creation to the legacy YUI popup.
       * 
       * @instance
       * @param {string} site
       */
      createProject: function lyse_ProjectService_createProject(config) {
         // TODO: When an edit site request is received we should display the edit site dialog.
         //       We need to wrap the existing YUI widget in a Dojo object.
         this.alfLog("log", "A request has been made to create a project: ", config);
         
         // Just use the old YUI popup...
         if (Alfresco && Alfresco.module && typeof Alfresco.module.getCreateProjectInstance === "function")
         {
            Alfresco.module.getCreateProjectInstance().show();
         }
         else
         {
            this.alfLog("error", "Could not find the 'Alfresco.module.getCreateProjectInstance' function - has 'lyse-create-project.js' been included in the page?");
         }
      },
      /**
       * This method delegates site creation to the legacy YUI popup.
       * 
       * @instance
       * @param {string} site
       */
      createContractor: function lyse_ProjectService_createContractor(config) {
         // TODO: When an edit site request is received we should display the edit site dialog.
         //       We need to wrap the existing YUI widget in a Dojo object.
         this.alfLog("log", "A request has been made to create a contractor: ", config);
         
         // Just use the old YUI popup...
         if (Alfresco && Alfresco.module && typeof Alfresco.module.getCreateContractorInstance === "function")
         {
            Alfresco.module.getCreateContractorInstance(config).show();
         }
         else
         {
            this.alfLog("error", "Could not find the 'Alfresco.module.getCreateContractorInstance' function - has 'lyse-create-contractor.js' been included in the page?");
         }
      },
      /**
       * This method delegates site editing to the legacy YUI popup.
       * 
       * @instance
       * @param {string} site
       */
      editProject: function lyse_ProjectService_editProject(config) {
         // TODO: When an edit site request is received we should display the edit site dialog.
         //       We need to wrap the existing YUI widget in a Dojo object.
         this.alfLog("log", "A request has been made to edit a site: ", config);
         
         // Just use the old YUI popup...
         if (Alfresco && Alfresco.module && typeof Alfresco.module.getEditProjectInstance === "function")
         {
           var test = Alfresco.module.getEditProjectInstance();

           Alfresco.module.getEditProjectInstance().show({
               shortName: config.site
            });
         }
         else
         {
            this.alfLog("error", "Could not find the 'Alfresco.module.getEditProjectInstance' function - has 'edit-site.js' been included in the page?");
         }
      },
      /**
       * This method delegates site editing to the legacy YUI popup.
       * 
       * @instance
       * @param {string} site
       */
      editContractor: function lyse_ProjectService_editContractor(config) {
         // TODO: When an edit site request is received we should display the edit site dialog.
         //       We need to wrap the existing YUI widget in a Dojo object.
         this.alfLog("log", "A request has been made to edit a site: ", config);
         
         // Just use the old YUI popup...
         if (Alfresco && Alfresco.module && typeof Alfresco.module.getEditContractorInstance === "function")
         {
           var test = Alfresco.module.getEditContractorInstance();

           Alfresco.module.getEditContractorInstance().show({
               shortName: config.site
            });
         }
         else
         {
            this.alfLog("error", "Could not find the 'Alfresco.module.getEditContractorInstance' function - has 'edit-site.js' been included in the page?");
         }
      }   
  });
});